//!OpenSCAD
Width = 1;
Length = 4;
Thickness = 0.216;
FoldThickness = 0.072;
KerfSpacing = 0.2;
Number_of_Kerfs = 10;
Vangle = 30;
EndmillDiameter = 0.25;
Generate_DXF = true;
Keft_Test = "Spacing";
$fn = 80;

endmillradius = EndmillDiameter / 2;

module maketestpiecearray() {
  if (Keft_Test == "Number") {
    for (j = [0 : abs(1) : Number_of_Kerfs - 1]) {
      translate([(j * (Width + EndmillDiameter * 1.1)), 0, 0]){
        maketestpiece(Width, Length, Thickness, Vangle, FoldThickness, KerfSpacing, j + 1);
      }
    }

  } else {
    for (k = [0 : abs(1) : Number_of_Kerfs - 1]) {
      translate([(k * (Width + EndmillDiameter * 1.1)), 0, 0]){
        maketestpiece(Width, Length, Thickness, Vangle, FoldThickness, (KerfSpacing / Number_of_Kerfs) * (k + 1), Number_of_Kerfs);
      }
    }

  }

}

module maketestpiece(bswidth, bslength, bsthickness, bsangle, bsfoldthickness, bsspacing, numkerfs) {
  if (Generate_DXF == true) {
    union(){
      cube([bswidth, bslength, bsthickness], center=false);
      translate([0, (bslength + EndmillDiameter * 3.3), (-Thickness)]){
        for (i = [0 : abs(1) : numkerfs - 1]) {
          translate([0, ((bslength - (tan((bsangle / 2)) * (bsthickness - bsfoldthickness) + bsspacing) * (numkerfs - 1)) / 2), bsfoldthickness]){
            translate([0, (i * (tan((bsangle / 2)) * (bsthickness - bsfoldthickness) + bsspacing)), 0]){
              hull(){
                vendmill();
                translate([bswidth, 0, 0]){
                  vendmill();
                }
              }
            }
          }
        }

      }
    }
  } else {
    difference() {
      cube([bswidth, bslength, bsthickness], center=false);

      for (i = [0 : abs(1) : numkerfs - 1]) {
        translate([0, ((bslength - (tan((bsangle / 2)) * (bsthickness - bsfoldthickness) + bsspacing) * (numkerfs - 1)) / 2), bsfoldthickness]){
          translate([0, (i * (tan((bsangle / 2)) * (bsthickness - bsfoldthickness) + bsspacing)), 0]){
            hull(){
              vendmill();
              translate([bswidth, 0, 0]){
                vendmill();
              }
            }
          }
        }
      }

    }
  }

}

module makearrayoutline() {
  translate([(-(EndmillDiameter * 1.1)), (-(EndmillDiameter * 1.1)), 0]){
    hull(){
      cylinder(r1=endmillradius, r2=endmillradius, h=FoldThickness, center=false);
      translate([(EndmillDiameter * 1.1 + Number_of_Kerfs * (Width + EndmillDiameter * 1.1)), 0, 0]){
        cylinder(r1=endmillradius, r2=endmillradius, h=FoldThickness, center=false);
      }
      translate([endmillradius, (Length + EndmillDiameter * 2.2), 0]){
        cylinder(r1=0, r2=endmillradius, h=FoldThickness, center=false);
      }
      translate([(EndmillDiameter * 1.1 + Number_of_Kerfs * (Width + EndmillDiameter * 1.1)), (Length + EndmillDiameter * 2.2), 0]){
        cylinder(r1=endmillradius, r2=endmillradius, h=FoldThickness, center=false);
      }
    }
  }
}

module vendmill() {
  cylinder(r1=0, r2=endmillradius, h=(endmillradius / tan((Vangle / 2))), center=false);
}

if (Generate_DXF == true) {projection(cut = true) {
  difference() {
    union(){
      makearrayoutline();
      translate([0, (Length + EndmillDiameter * 3.3), 0]){
        makearrayoutline();
      }
    }

    maketestpiecearray();
  };}
} else {
  maketestpiecearray();
}
